/*
 * Copyright (C) 2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

package estella.lib;
import  estella.lib.Constantes;
import  estella.lib.Tempo;

import java.io.IOException;
import java.io.RandomAccessFile;

/**
* Funcoes para gerenciar o log, que eh gravado sempre em um arquivo, com um nome definido ou nao pelo usuario.
* Caso o usuario nao defina o nome, sera usado o nome que estiver na classe de Constantes.
* @author Fabio Generoso
* @since 20/11/2005
*/
public class Log
{
  private Constantes CON;
  private Tempo tempo;
  private static RandomAccessFile log;
  private static RandomAccessFile log_erro;
	private String cab;
	
	/**
	* Define a criacao de um novo log
	* @param cab String com o cabecalho do log
	* @see Constantes
	*/
	public Log(String cab)
	{
		CON = new Constantes();
		criar(CON.LOG);
		cabecalho(cab);
	}
	
	/**
	* Define a criacao de um novo log
	* @see Constantes
	*/
	public Log()
	{
		cab = new String("");
		CON = new Constantes();
		criar(CON.LOG);
	}
	
	/**
	* Define a criacao de um novo log com o respectivo nome passado pelo usuario.
	* @param log_name nome do log (arquivo)
	*/
	public Log(String log_nome, String cab)
	{
		criar(log_nome);
		cabecalho(cab);
	}
	
	/**
	* Seta um cabecalho no log, todo  log adicionado tera esse cabecalho
	* @param cab string com o cabecalho
	* @return true ou false se correamente setado
	*/
	public boolean cabecalho(String cab)
	{
		try {
			this.cab = new String(cab);
			return true;
		}
		catch(Exception FalhaCabecalho) {
			FalhaCabecalho.printStackTrace();
			return false;
		}
	}

	/**
	* Cria um novo arquivo de log
	* @param log_nome nome do log
	* @return nenhum
	*/
	private void criar(String log_nome)
	{
		tempo = new Tempo();
		try {
			log = new RandomAccessFile(log_nome, "rw");
			log.seek(log.length());
		}
		catch(IOException IO_LOG) {
			IO_LOG.printStackTrace();
		}
	}

	/**  
	* Adiciona uma mensagem com data e hora no arquivo de log
	* @param mensagem (string)
	* @param tipo tipo da mensagem de log (atencao, info, etc).
	* @param Tempo Classe com funcoes de tempo
	* @see Tempo
	*/
	private boolean adicionar(String tipo, String msg)
	{
		System.out.println("----------------------------------------------------------------");
		System.out.print(cab + "(" + tempo.data() + "  " + tempo.hora() + ") " + tipo + "-> " + msg + "\n");
		
		msg = cab + ";" + tempo.data() + ";" + tempo.hora() + ";" + tipo + ";" + msg + "\n";
		try {
			synchronized (log) {
				log.write(msg.getBytes());
			}
			return true;
		}
		catch(IOException IO_LOG) {
			IO_LOG.printStackTrace();
			return false;
		}
	}
	
	/**
	*  Adiciona uma nova mensagem de informacao no log
	* @param msg_info String com a mensagem
	* @return true ou false
	*/
	public boolean info(String msg_info)
	{
		return (adicionar("Info", msg_info));
	}
	
	/**
	*  Adiciona uma nova mensagem de erro no log
	* @param msg_info String com a mensagem
	* @return true ou false
	*/
	public boolean erro(String msg_erro)
	{
		return (adicionar("Erro",msg_erro));
	}
	
	/**
	*  Adiciona uma nova mensagem de erro no log
	* @param msg_info String com a mensagem
	* @return true ou false
	*/
	public boolean falha(String msg_falha)
	{
		return (adicionar("Falha", msg_falha));
	}
 
  /**
	*  Adiciona uma nova mensagem de atencao (warning) no log
	* @param msg_info String com a mensagem
	* @return true ou false
	*/
	public boolean atencao(String msg_atencao)
	{
		return (adicionar("Atencao", msg_atencao));
	}
  
  /**
  * Fechar - fecha o sistema de log
  * @return true ou false
  */
  public boolean fecha()
  {
    try {
      log .close();
      return true;
    }
    catch(IOException FecharLog) {
      FecharLog.printStackTrace();
      return false;
    }
  }
}
